# -*- coding: utf-8 -*-
from datetime import date
from _version import __version__
import requests


class Font:
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'


def version_to_tuple(version:str) -> tuple:
    return tuple(map(int, version.split('.')))


# Get the latest released version of the package
try:
    data = requests.get('https://pypi.org/pypi/mcwpy/json').json()
except:
    print(f'{Font.FAIL}Unable to resolve latest release version.{Font.ENDC}')
    quit()
else:
    print(f'{Font.OKGREEN}Latest released version: {data["info"]["version"]}.{Font.ENDC}')
del requests

# Verify that the version is the latest
if version_to_tuple(data['info']['version']) < version_to_tuple(__version__):
    print(f'{Font.OKGREEN}New version: {__version__}.{Font.ENDC}')
else:
    print(f"{Font.FAIL}New version ({__version__}) <= older version ({data['info']['version']}).{Font.ENDC}")
    quit()

# Verify that the CHANGELOG is up to date
if f"# {__version__} ({str(date.today()).replace('-', '/')})" in open('CHANGELOG.md').read():
    print(f'{Font.OKGREEN}CHANGELOG.md is up to date.{Font.ENDC}')
else:
    print(f'{Font.FAIL}Unable to read CHANGELOG.md or missing version details.{Font.ENDC}')
    quit()
del date

# Verify that the CHANGELOG lines start with a hyphen and end with a period and a double space.
lines = [line for line in open('CHANGELOG.md').readlines() if (line.startswith('-') or not line.startswith('#')) and not line.endswith('.  \n') and not line == '\n']
if len(lines) == 0:
    print(f'{Font.OKGREEN}All lines in CHANGELOG.md seem to start with a hyphen and end with a period and a double space.{Font.ENDC}')
else:
    print(f'{Font.FAIL}{len(lines)} {"line" if len(lines) < 2 else "lines"} should start with a hyphen and end with a period and a double space: {lines}.{Font.ENDC}')
    quit()

print(f'{Font.OKGREEN}\n>>> Setup check passed successfuly. <<<{Font.ENDC}')



# -*- coding: utf-8 -*-
from datetime import date
from time import time
from workspace import Workspace
import os


class Datapack:
    """
    Datapacks can be placed in the .minecraft/saves/(world)/datapacks folder of a world. Each data pack is either a sub-folder or a .zip file \
        within the datapacks folder. After it is in the folder, a data pack is enabled for that world when the world is reloaded or loaded.
    Data packs load their data based on the load order. This order can be seen and altered by using the /datapack command and is stored in the \
        level.dat file.

    The player can also select data packs at the world creation screen by clicking the Data Packs button and dragging-and-dropping their data \
        pack folders/zip-files there. This is similar to the Resource Pack selection screen, and allows the player to enable data packs before \
        the world is generated, and easily customize the load order too.
    """
    def __init__(self, 
                 title: str=None,
                 path: str=None,
                 author: str=None,
                 pack_mcmeta: dict=None,
                 workspaces: list=None,
                 auto_compile: bool=None,
                 compile_as_zip: bool=None,
                 replace_existing: bool=None,
                 description: str=None,
                 version: str=None
                 ) -> None:
        """
        Initialize a new Datapack object which will then generate a Minecraft Datapack.

        :param title: The title of the datapack.
        :param path: The path to the datapack.
        :param author: The author of the datapack.
        :param pack_mcmeta: The metadata of the datapack.
        :param workspaces: The workspace(s) in the datapack.
        :param auto_compile: Whether or not to automatically compile the datapack.
        :param compile_as_zip: Whether or not to compile the datapack as a zip file.
        :param replace_existing: Whether or not to replace an existing datapack with the same name.
        :param description: A short description of the datapack.
        :param version: The version of the datapack.
        :return: None; this is a constructor.
        """
        self.title = title if title not in [None, ''] else "My_Amazing_Datapack"
        self.path = path if path is not None else os.getcwd()
        self.author = author if author is not None else "MCWPy"
        self.pack_mcmeta = pack_mcmeta if pack_mcmeta is not None else {}
        self.workspaces = (workspaces if isinstance(workspaces, list) else [workspaces]) if workspaces is not None else []
        self.auto_compile = auto_compile if auto_compile is not None else False
        self.compile_as_zip = compile_as_zip if compile_as_zip is not None else False
        self.replace_existing = replace_existing if replace_existing is not None else False
        self.description = description if description is not None else ""
        self.version = version if version is not None else f'{str(date.today().isocalendar()[0])[-2:]}w{date.today().isocalendar()[1]}s{int(time())}'

        # Verifies that the path is a valid datapack path
        self._exists = os.path.exists(self.path + self.title)

        if self._exists and not self.replace_existing:
            self.replace_existing = input(f'{self.title} already exists, do you want to replace it? [yes/no]: ')[0].lower() == 'y'

            if not self.replace_existing:
                raise FileExistsError(f'The datapack "{self.title}" already exists. Please specify a new name or set the "replace_existing" parameter to True.')

        # Verifies that the workspaces are valid.
        if not all(isinstance(w, Workspace) for w in self.workspaces):
            raise TypeError(f'The "workspaces" parameter must be a list of Workspace objects.')

        # Auto-compile?
        if self.auto_compile:
            self.compile()


    def compile(self) -> None:
        pass                

    def __str__(self) -> str:
        ws_sep = ', \n\t\t'
        return f"---- {self.title}\n\t|\n\t---- pack.mcmeta: {self.pack_mcmeta}\n\t---- pack.png\n\t---- data\n\t\t|\n\t\t{ws_sep.join(map(lambda x: f'---- {x.name}', self.workspaces))}"

    def __repr__(self) -> str:
        return self.__str__()

    def __iter__(self) -> iter:
        return Datapack_Iterator(self.workspaces)

    def __len__(self) -> int:
        return len(self.workspaces)

class Datapack_Iterator:
    """Iterator class for Datapack"""
    def __init__(self, workspaces) -> None:
        """
        Initialize a new Datapack_Iterator object which will iterate over the workspaces in a Datapack.

        :param workspaces: The list of workspaces to iterate through.
        """
        self._workspaces = workspaces
        self._index = 0

    def __next__(self) -> Workspace:
        """Select the next workspace in the Datapack until the end is reached."""
        if self._index < len(self._workspaces):
            result = self._workspaces[self._index]
            self._index += 1
            return result
        raise StopIteration


if __name__ == '__main__':
    """
    This part is just for testing purposes.
    """
    dp = Datapack(workspaces=[Workspace(name='string0'), Workspace(name='string1')])
    for e in dp:
        print(e.name)
    print(dp)



# -*- coding: utf-8 -*-
""" MCWPy - Minecraft With Python

MCWPy is a Python library for creating Minecraft datapacks using Python.
    https://github.com/vianneyveremme/minecraft_with_python
"""
from .ansi_escape_sequences import Font
from .datapack import Datapack
from .workspace import Workspace
import sys


# Checking Python version (should be above 3.9.5)
if sys.version_info < (3, 9, 5):
    print(f"{Font.WARNING}For optimal results it is recommended to use Python 3.9.5 or above.{Font.ENDC}")
else:
    print(f"{Font.HEADER}{Font.BOLD}Hello from the MCWPy community :){Font.ENDC}")



# -*- coding: utf-8 -*-
from datapack import Datapack
from datetime import date
from time import time
from workspace import Workspace
import os
import unittest


class TestStringMethods(unittest.TestCase):
    def test_datapack_default_values(self):
        self.assertEqual(Datapack().title, 'My_Amazing_Datapack')
        self.assertEqual(Datapack().path, os.getcwd())
        self.assertEqual(Datapack().author, 'MCWPy')
        self.assertEqual(Datapack().pack_mcmeta, {})
        self.assertEqual(Datapack().workspaces, [])
        self.assertFalse(Datapack().auto_compile)
        self.assertFalse(Datapack().compile_as_zip)
        self.assertFalse(Datapack().replace_existing)
        self.assertEqual(Datapack().description, '')

    def test_datapack_default_instances(self):
        self.assertIsInstance(Datapack(), Datapack)
        self.assertIsInstance(Datapack().title, str)
        self.assertIsInstance(Datapack().path, str)
        self.assertIsInstance(Datapack().author, str)
        self.assertIsInstance(Datapack().pack_mcmeta, dict)
        self.assertIsInstance(Datapack().workspaces, list)
        self.assertIsInstance(Datapack().auto_compile, bool)
        self.assertIsInstance(Datapack().compile_as_zip, bool)
        self.assertIsInstance(Datapack().replace_existing, bool)
        self.assertIsInstance(Datapack().description, str)

    def test_datapack_values_set(self):
        self.assertEqual(Datapack(title='string').title, 'string')
        self.assertEqual(Datapack(path='string').path, 'string')
        self.assertEqual(Datapack(author='string').author, 'string')
        self.assertEqual(Datapack(pack_mcmeta={'string': 'string'}).pack_mcmeta, {'string': 'string'})
        self.assertEqual(Datapack(workspaces=[0, 1]).workspaces, [0, 1])
        self.assertTrue(Datapack(auto_compile=True).auto_compile)
        self.assertTrue(Datapack(compile_as_zip=True).compile_as_zip)
        self.assertTrue(Datapack(replace_existing=True).replace_existing)
        self.assertEqual(Datapack(description='string').description, 'string')
        self.assertEqual(Datapack(version='string').version, 'string')

    def test_datapack_iteration_with_workspaces(self):
        self.assertEqual([e.name for e in Datapack(workspaces=[Workspace(name='string'), Workspace(name='string')])], ['string', 'string'])


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
- Use 'python3 setup.py sdist' in a terminal to create the package.  
- Use 'twine upload --repository-url https://upload.pypi.org/legacy/ dist/*' to upload the package on pypi.  
"""
from setuptools import setup, find_packages
from _version import __version__


classifiers = [
    'Development Status :: 1 - Planning',
    'Intended Audience :: Customer Service',
    'Intended Audience :: Developers',
    'Intended Audience :: Education',
    'Intended Audience :: End Users/Desktop',
    'Operating System :: OS Independent',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.9'
    
]

setup(
    author='Vianpyro',
    author_email='vianney.veremme@gmail.com',
    classifiers=classifiers,
    description='A new way to create Minecraft datapacks',
    install_requires=open('REQUIREMENTS.txt').read(),
    keywords=['Minecraft', 'Datapack', 'Function', 'MCFunction'],
    license='MIT',
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.md').read(),
    long_description_content_type='text/markdown',
    name='mcwpy',
    packages=find_packages(exclude=['tests']),
    py_modules=['ansi_escape_sequences', 'datapack', 'datapack', 'test'],
    url='https://github.com/vianneyveremme/minecraft_with_python',
    version=__version__,
    zip_safe=False
)
